<?php
/**
 * Created by PhpStorm.
 * User: Roman Steinberg
 * Date: 29.01.2019
 * Time: 15:55
 */

abstract class Model_Collection extends Model implements JsonSerializable
{
    protected $data;
    protected $parameters;
    protected $queryName;

    /**
     * @var bool
     * Indicates whether data was loaded from db.
     */
    protected $loaded = false;

    /**
     * @var bool
     * Allow to load data from database, if current
     * object instance does not contains some fields.
     */
    protected $lazy = true;

    /**
     * @param $data        array
     *                     Source data for building the object.
     * @param $lazyLoading bool
     *                     Indicates whether lazy loading can be used.
     */
    public function __construct($lazyLoading = true) {
        $this->data = [];
        $this->lazy = $lazyLoading;
        $this->setQuery();
    }

    /**
     * @return array
     *                  Data selection
     */
    private function getRawData() {
        // validation
        if (!isset($this->queryName) or !isset($this->parameters)) {
            throw new LogicException(Error::INVALID_PARAMETERS);
        }

        // assemble query
        $sql = /** @lang text */
            'SELECT * FROM '.$this->queryName.'(';
        foreach ($this->parameters as $param => $value) {
            $sql = $sql.$param.', ';
        }
        $sql = substr($sql, 0, -2).')';

        // execute
        $q = DB::query(Database::SELECT, $sql);
        foreach ($this->parameters as $param => $value) {
            $q->param($param, $value);
        }
        return $q->execute()->as_array();
    }

    protected function getRawDataInLazyMode() {
        if (!$this->loaded && $this->lazy) {
            $this->data = static::getRawData();
            $this->loaded = true;
        }
    }

    public function jsonSerialize($forceLoad = true) {
        if ($forceLoad)
            $this->getRawDataInLazyMode();
        return $this->data;
    }

    /**
     * Sets query name and parameters for getting collection
     * @return null
     */
    protected abstract function setQuery();

    /**
     * Set parameters values for collection getting.
     * @return $this
     */
    public function setParamValues($paramValues) {

        foreach ($paramValues as $param => $value) {
            // validation
            if (!array_key_exists($param, $this->parameters))
                throw new LogicException(Error::INVALID_PARAMETERS);

            // check that data need to be loaded
            if ($this->parameters[$param] !== $value)
                $this->loaded = false;

            // set parameter value
            $this->parameters[$param] = $value;
        }

        return $this;
    }
}